from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, Contains, NotEquals
from autopilot.input import Mouse, Pointer
from time import sleep
import os
import shutil
import tempfile

class FileRollerTests(AutopilotTestCase):
    '''Collection of autopilot test for file-roller (Archive Manager) '''

    def setUp(self):
        '''Set-up method'''
        super(FileRollerTests, self).setUp()
        self.app = self.launch_test_application('file-roller')

        # only instantiate these once for multiple tests
        # these can only be instantiated here if you do not need to reload the application during a test
        # As the Gtk id's change when application is relaunched. and throws error
        # So for tests that are reloaded also grab new Gtkobject id's after relaunch
        self.pointing_device = Pointer(Mouse.create())
        self.archiveWindow = self.app.select_single('FrWindow')
        self.add_files_button = self.app.select_single('GtkLabel', label=u'Add Files')
        self.extract_button = self.app.select_single('GtkLabel', label=u'Extract')
        self.open_button = self.app.select_single('GtkLabel', label=u'Open')
        self.new_archive_button = self.app.select_single('GtkToolButton', name=u'New')
        
    def test_create_archive(self):
        '''Creates an archive and adds sample data, Window title must contain archive name'''
        
        tempArchive = _tempArchiveName()
        #Test: Archive must be created with sample data
        self.create_archive_with_sample_data(tempArchive)
        # If create archive successfull, then check here that the window title
        # is the name of the archive.
        (dirName, fileName) = os.path.split(tempArchive)
        self.assertThat(self.archiveWindow.title, Eventually(Contains(fileName)))
        
        #Cleanup
        self.delete_temp_archive()
        
        
        
    def test_open_archive(self):
        '''Open archive window title must contains the archive name '''
        
        tempArchive = _tempArchiveName()
        # Call create and relaunch to get a safe way of reloading an application
        self.create_and_relaunch(tempArchive)
               
        #Test: Must open archive
        self.open_test_archive(tempArchive)
        # if open archive successful, then check here that the window title
        # is the name of the archive.
        (dirName, fileName) = os.path.split(tempArchive)
        self.assertThat(self.archiveWindow.title, Eventually(Contains(fileName)))
        # Cleanup
        self.delete_temp_archive()
        
    def test_extract_archive(self):
        '''Open archive and extract file'contents,'''
        
        tempArchive = _tempArchiveName()
        # Call create and relaunch to get a safe way of reloading an application
        self.create_and_relaunch(tempArchive)
        self.assertThat(self.archiveWindow.title, Eventually(Contains('Archive Manager')))
        #Test: Open and extract an archive
        self.extract_test_archive(tempArchive)
        ## assert we have come back to main window after extraction
        ## Test Will not get back to mainWindow if extraction failed.
        ## Check window title is same as archive
        (dirName, fileName) = os.path.split(tempArchive)
        self.assertThat(self.archiveWindow.title, Eventually(Contains(fileName)))
        #Cleanup
        self.delete_temp_archive()
        self.delete_temp_directory(fileName)
        
    # this method needs some more assertions. At the moment its relying that commands are correct
    def create_archive_with_sample_data(self, tempArchive):
        ''' Must create archive and add sample data'''
        #Click on the new archive button
        self.pointing_device.move_to_object(self.new_archive_button)
        self.pointing_device.click()
        # The new archive dialog window title must have 'New Archive'
        self.new_dialog = self.app.select_single('FrNewArchiveDialog')
        self.assertThat(self.new_dialog.title, Eventually(Equals(u'New Archive')))

        #Enter name of new archive
        self.keyboard.type(tempArchive)

        # TODO: Figure out a gzip commpress, does not have an active Id enabled

        # select the create button and check it is clickable
        self.create_button = self.app.select_single('GtkButton', label=u'create-archive')
        self.assertThat(self.create_button.label, Eventually(NotEquals(None)))
        # Click on Create button
        self.pointing_device.move_to_object(self.create_button)
        self.pointing_device.click()
        # Click on the add files button
        self.pointing_device.move_to_object(self.add_files_button)
        self.pointing_device.click()
        # The add files dialog title must contain 'Add Files'
        self.add_files_dialog = self.app.select_single('FrFileSelectorDialog')
        self.assertThat(self.add_files_dialog.title, Eventually(Equals('Add Files')))

        # This is a horrible hacky workaround to add sample files from /usr/share/example-content/*
        # Need to find a better workaround :S Location GtkEntry doesnt work when entering file path
        # Maybe add some assertions to check that the Location GTKEntry contains correct path
        self.keyboard.press_and_release("Alt+l")
        self.keyboard.press_and_release("Tab")
        self.keyboard.press_and_release("Tab")
        self.keyboard.type("File")
        self.keyboard.press_and_release("Enter")
        self.keyboard.press_and_release("Tab")
        self.keyboard.press_and_release("Right")
        self.keyboard.press_and_release("Right")
        self.keyboard.type("usr")
        self.keyboard.press_and_release("Enter")
        self.keyboard.type("share")
        self.keyboard.press_and_release("Enter")
        self.keyboard.type("example-content")
        self.keyboard.press_and_release("Enter")
        self.keyboard.press_and_release("Left")
        self.keyboard.press_and_release("Left")
        self.keyboard.press_and_release("Enter")
        # We must be able to click the '_Add' button
        self.add_folder_button = self.app.select_single('GtkLabel', label=u'_Add')
        self.assertThat(self.add_folder_button.label, Eventually(NotEquals(None)))
        #Click the '_Add" button
        self.pointing_device.move_to_object(self.add_folder_button)
        self.pointing_device.click()
        # sleep needed to give sample data enough time to be added
        sleep(10)
        
    # Method for opening an archive, needs assertions adding
    def open_test_archive(self, tempArchive):
        '''Open the test archive'''
        self.open_button = self.app.select_single('GtkLabel', label=u'Open')
        self.pointing_device.move_to_object(self.open_button)
        self.pointing_device.click()

        self.keyboard.press_and_release("Alt+l")
        self.keyboard.type(tempArchive)
        

        self.open_archive_button = self.app.select_single('GtkLabel', label=u'_Open')
        self.pointing_device.move_to_object(self.open_archive_button)
        self.pointing_device.click()

    def extract_test_archive(self, tempArchive):
        '''Open Archive and extract the sample data'''
        
        self.open_test_archive(tempArchive)
        (dirName, fileName) = os.path.split(tempArchive)

        self.assertThat(self.archiveWindow.title, Eventually(Contains(fileName)))

        # Check that the Extract button is now clickable
        self.assertThat(self.extract_button.label, Eventually(NotEquals(None)))

        #click on extract button
        self.pointing_device.move_to_object(self.extract_button)
        self.pointing_device.click()

        self.keyboard.press_and_release("Alt+l")
        self.keyboard.type(tempArchive)

        self.extract_archive_button = self.app.select_single('GtkLabel', label=u'_Extract')
        self.assertThat(self.extract_archive_button.label, Eventually(NotEquals(None)))

        self.pointing_device.move_to_object(self.extract_archive_button)
        self.pointing_device.click()
        # need this sleep to give enough time for the GtkDialog time to load before
        # we can select it
        sleep(10)
        self.extract_dialog = self.app.select_single('GtkDialog')
        self.assertThat(self.extract_dialog.title, Eventually(Contains(u"")))
        self.show_files_button = self.app.select_single('GtkLabel', label=u'_Show the Files')
        self.assertThat(self.show_files_button.label, Eventually(NotEquals(None)))

        self.pointing_device.move_to_object(self.show_files_button)
        self.pointing_device.click()
        # This sleep is needed as we can't introspect nautilus when it shows the files
        # TODO: Once nautilus can be introspected we need to lose this sleep
        # and confirm nautilus window title Eventually Contains fileName
        sleep(10)

        self.keyboard.press_and_release("Alt+F4")
        
    def create_and_relaunch(self, tempArchive):
        ''' This method calls create archive_with_sample_data
            then will close the application and relaunch it 
            grabbing new Gtk object id's'''
        # Create archive and check window title
        self.create_archive_with_sample_data(tempArchive)
        (dirName, fileName) = os.path.split(tempArchive)

        self.assertThat(self.archiveWindow.title, Eventually(Contains(fileName)))
        # Close the Application
        self.keyboard.press_and_release('Ctrl+w')
        # launch a new instance of fire-roller
        self.app = self.launch_test_application('file-roller')
        # We need to get the new object id's here as we have reloaded the application and
        # will throw an error unknown id if we don't :)
        self.archiveWindow = self.app.select_single('FrWindow')
        self.add_files_button = self.app.select_single('GtkLabel', label=u'Add Files')
        self.extract_button = self.app.select_single('GtkLabel', label=u'Extract')
        self.open_button = self.app.select_single('GtkLabel', label=u'Open')
        self.new_archive_button = self.app.select_single('GtkToolButton', name=u'New')
    
    def delete_temp_archive(self):
        # Cleanup
        temp = self.archiveWindow.title
        # Strip archive name of whitespace
        temp = temp.strip()
        path = os.path.join('/tmp/'+temp)
        self.addCleanup(os.unlink, path)
        
    def delete_temp_directory(self, fileName ):
        #Delete the extracted directory
        extract_path = os.path.join('/tmp/'+ fileName)
        self.addCleanup(shutil.rmtree, extract_path)
    
    
#hardcoded global archiveName
def _tempArchiveName():
    '''Temporary archive name for testing with.'''
    sFile = tempfile.NamedTemporaryFile()
    sFile.close()
    return sFile.name

